#ifndef __GLVIEW_HPP__
#define __GLVIEW_HPP__

#include <QtGui>
#include <QOpenGLWidget>

class GLview : public QOpenGLWidget, protected QOpenGLFunctions  {
  Q_OBJECT
public:
  GLview(QWidget *parent = 0);
  ~GLview() { 
  }
  void keyPressGL(QKeyEvent* e);
protected:
  void initializeGL(); // QGLWidget OpenGL interface
  void initCameraGL();
  void paintGL();
  void resizeGL(int width, int height);

  QMatrix4x4 model, view, projection; 

  QOpenGLShaderProgram phong_shader;

  float thetax, thetay, thetaz;
  
  // Camera parameters --------------
  QVector3D eye, lookCenter, lookUp; 
  QQuaternion camrot;
  float yfov, neardist, fardist;  
  // -------------------------------

  // Geometry data -----------------
  QOpenGLBuffer vertexBuffer, normalBuffer;
  int num_vertices;
  // ------------------------------

  // Light/shading parameters ------
  QVector3D LightDirection, LightIntensity;
  QVector3D Ka, Kd, Ks;
  float Ns;

  bool prepareShaderProgram(QOpenGLShaderProgram &shader, 
			    const QString &vertex_file, 
			    const QString &fragment_file);

  bool prepareShaderProgramTex();
  QOpenGLVertexArrayObject vao;

  // Used to track mouse position for keyboard shortcuts.
  bool lastPosFlag; // Flag is true if a last position has been captured. Set to false 
                    // prior to mouse driven interaction.
  float lastPosX, lastPosY;   // Mouse state information.
  bool scaleFlag, translateFlag, rotateFlag;

  void mousePressEvent(QMouseEvent *event); 
  void mouseMoveEvent(QMouseEvent *event);
  void toggleRotate() { 
    translateFlag = scaleFlag = false;
    rotateFlag = !rotateFlag; setMouseTracking(rotateFlag); lastPosFlag = false; 
  }
  void toggleScale() { 
    translateFlag = rotateFlag = false;
    scaleFlag = !scaleFlag; setMouseTracking(scaleFlag); lastPosFlag = false; 
  }
  void toggleTranslate() { 
    rotateFlag = scaleFlag = false;
    translateFlag = !translateFlag; setMouseTracking(translateFlag);  lastPosFlag = false; 
  }

  QElapsedTimer elapsed_time;
  double timeAccumulator, totalTime;
  void updateGLview(float dt);				   
  void timerEvent(QTimerEvent *event);  
};

#endif

